  /*
  Для WeMos, связанной с Исполнительным Устройством.
  Приём от Ардуино режима работы. 
  Выдача сигналов на излучатель.
  Обработка входного сигнала полностью и передача результатов на Ардуино.
  То же, что и V3, но с сериалом
  */

  #include <ledc.h>                       // Подключение библиотеки ШИМ
  #include <esp_now.h>
  #include <Arduino.h>                    // библиотека Arduino 
  #include <WiFi.h>
  #include <esp_wifi.h>
  #include <SPI.h>
  #include "MPU9250.h"
  #include <Adafruit_BMP280.h>
  #include <EEPROM.h>

                    // Новый MAC Address этой платы
  uint8_t newMACAddress[] = {0x30, 0x83, 0x98, 0xB2, 0x6E, 0xAF};    
 
 	 // МАС-АДРЕС платы, на которую отправляем данные. C8:C9:A3:6A:D9:64
  uint8_t broadcastAddress[] = {0xC8, 0xC9, 0xA3, 0x6A, 0xD9, 0x64};


  #define  EEPROM_SIZE      80            // определение количества ячеек EEPROM
  #define  STR_Pin          16            // назначение вывода STR
  #define  FO_Pin           18            // назначение вывода FO
  #define  DC_Pin           33            // назначение вывода DC
  #define  echoPinA         1             // назначение вывода echoA L4 (3500 mV)
  #define  echoPinB         2             // назначение вывода echoB L3 (2500 mV)
  #define  echoPinC         4             // назначение вывода echoC L2 (1000 mV)
  #define  echoPinD         6             // назначение вывода echoD L1 (200 mV)

  #define  A0R_Pin         39             // назначение вывода A0R
  #define  A1R_Pin         37             // назначение вывода A1R
  #define  analog1Pin       8             // назначение вывода analog1Pin Uпит излучателя

  #define  START_Pin        3
  #define  PUSK_Pin        35             // назначение вывода PUSK
  #define  POZ_Pin         10             // назначение вывода START

  int eeprom_address = 0;                 // назначение первого адреса EEPROM

  MPU9250 IMU(SPI,12);
  Adafruit_BMP280 bmp(5); 

			  // Вводим переменные для строба ШИМ
  volatile int pwmfrequency = 20000;      // 20 кГц - 4; 10 кГц - 8; 5 кГц - 16 импульсов.
  volatile int target_duty = 256;         // Целевая скважность, до неё доходит.
  uint32_t scale = 1;                     // Количество импульсов, на которое изменяется скважность.
  uint32_t cycle_num = 1;                 // Количество периодов, через которое будет достигнута целевая скважность.
  uint32_t dutyCycle = 255;               // Начальная скважность.
  uint32_t idle_level = 0;                // Уровень выходного сигнала при выключении фадинга.
  uint32_t freqCLK = 20000;               // Частота ШИМ
  uint32_t pozknnow;                      // Текущее машинное время в мкс
  uint32_t pozknpush;                     // время нажатия кнопки 2
  uint32_t starknnow;                     // Текущее машинное время в мкс
  uint32_t starknpush;                    // время нажатия кнопки 2

			 // Вводим переменные для частоты ШИМ
  const int ledPin0 = FO_Pin;             // GPIO18	FO_Pin
  const int ledChannel0 = 0;              // Канал 0
  const int resolution0 = 2;              // Разрешение 2 бит
  const int dutyCycle0 = 2;               // Скважность 2 
  const int dutyCycle1 = 0;               // Скважность 0
  const uint32_t freq0 = 80000;           // Частота выходного сигнала

  uint8_t sensorId;
  bool pozknt,pozknp, pozkn = 0;          // флаг текущего, прошлого состояния и нажатия кнопки Enter
  bool flpuskt,flpuskp = 0;               // флаг текущего, прошлого состояния Pusk
  bool flstart,flstarp,flstar = 0;        // флаг текущего, прошлого состояния и нажатия кнопки Start
  bool zapisy = 0;                        // флаг процесса записи на карту памяти
  bool flMag = 0;                         // флаг калибровки компаса
  bool flAccel = 0;                       // флаг калибровки акселерометра

  float  hxb, hyb, hzb, exb, eyb, ezb, axb, ayb, azb, sxb, syb, szb = 0;
  float  hxs, hys, hzs, exs, eys, ezs, axs, ays, azs, sxs, sys, szs = 1;
  float  analog1 = 0.0;                   // объявление переменной analog1
  float  Direction;
  const  float Pi = 3.14159;
  uint32_t start;                         // Время старта
  uint32_t now;                           // Текущее машинное время в мкс
  uint16_t durationA;                     // объявление переменной длительности сигнала от приёмника
  uint16_t durationB;                     // объявление переменной длительности сигнала от приёмника
  uint16_t durationC;                     // объявление переменной длительности сигнала от приёмника
  uint16_t durationD;                     // объявление переменной длительности сигнала от приёмника
  uint32_t prevUsA;                       // Предыдущее время изменения состояния пина
  uint32_t prevUsB;                       // Предыдущее время изменения состояния пина
  uint32_t prevUsC;                       // Предыдущее время изменения состояния пина
  uint32_t prevUsD;                       // Предыдущее время изменения состояния пина
  uint16_t poz = 0;                       // позиция

  int l = 0;                              // номер канала
  int p = 0;                              // количество импульсов
  int u = 0;                              // усиление
  int countA = 0;                         // Счётчик циклов канала A
  int countB = 0;                         // Счётчик циклов канала B
  int countC = 0;                         // Счётчик циклов канала C
  int countD = 0;                         // Счётчик циклов канала D
  bool prevSA,flagA = 0;                  // флаг не первого измерения, флаг ошибки канала A
  bool prevSB,flagB = 0;                  // флаг не первого измерения, флаг ошибки канала B
  bool prevSC,flagC = 0;                  // флаг не первого измерения, флаг ошибки канала C
  bool prevSD,flagD = 0;                  // флаг не первого измерения, флаг ошибки канала D
  bool flagrec = 0;                       // флаг приёма данных

  #define LOG_SIZE 10                     // размер лога (LOG_SIZE ячеек)
  #define CHN_SIZE 4                      // размер лога (CHN_SIZE массивов каналов)
  #define GEO_SIZE 8                      // размер лога (GEO_SIZE массивов геоданных)
  uint8_t regout = 0x3A;                  // регистр выдачи на исполнительное устройство
  uint16_t dursig;                        // длительность сигнала от приёмника
  uint16_t us[CHN_SIZE][LOG_SIZE+6];      // массив измерений. Для длительности отрезков между изменениями входа

			 // Вводим переменные для хранения отправляемых данных
  uint8_t regch = 0;                      // регистр обмена данными
  float voltage1 = 0.0;                   // объявление переменной voltage1 
  uint16_t durus[CHN_SIZE][LOG_SIZE+6];   // массив измерений. Для текущего времени в момент изменения
  float geo[GEO_SIZE];                    // массив измерений. Для геоданных

          // Вводим переменные для хранения принимаемых данных
  uint8_t  incomingReg1;
 
         // Переменная для хранения состояния отправки
  String success;
 
                  // структура для приёма данных
        // (Должна совпадать со структурой передачи на плате-приемнике)
  typedef struct struct_recmes {
  uint8_t reg1;
  } struct_recmes;

                 // структура для отправки данных
         // (Должна совпадать со структурой приёма на плате-приемнике)
  typedef struct struct_sendmes {
  uint8_t regin;                          // регистр обмена данными
  float voltbat;                          // напряжение батареи
  uint16_t arrey1[CHN_SIZE][LOG_SIZE+6];  // массив измерений. 
  float arrey2[GEO_SIZE];                 // массив измерений. Для геоданных
  } struct_sendmes;
 
         // Создаем переменную для хранения отправляемого сообщения
  struct_sendmes ISPReadings;
 
               // То же, но для принимаемого сообщения
  struct_recmes incomingReadings;

        // Callback-функция для получения состояния отправки
  esp_now_peer_info_t peerInfo;

  void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  //Serial.print("\r\nLast Packet Send Status:\t");
  //Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
  if (status ==0){
    success = "Delivery Success :)";
  } else {
    success = "Delivery Fail :(";
  }
  }

            // То же, для индикации состояния приема данных
  void OnDataRecv(const uint8_t * mac, const uint8_t *incomingData, int len) {
  memcpy(&incomingReadings, incomingData, sizeof(incomingReadings));
  //Serial.print("Bytes received: ");
  //Serial.println(len);

                          // приём данных
  regout = incomingReadings.reg1;
  // printIncomingReadings();
  initial();
  flagrec = 1;                          // устанавливаем в 1 флаг приёма данных 
  }

  void printIncomingReadings(){
          // Отображаем показания в мониторе порта
  Serial.println("INCOMING READINGS"); 
  Serial.print("regout: ");
  Serial.println(regout, BIN);
  Serial.println(" ");
  }

  void printSendingData(){
  Serial.println("SENDING_DATA");
  Serial.print("regch: ");
  Serial.println(regch, BIN);
  Serial.print("voltage1: ");
  Serial.println(voltage1);
  for (int l = 0; l < CHN_SIZE; l++) {           // перебираем каналы
  Serial.print("Канал "); 
  Serial.println(l);
  Serial.println("durus: ");
  for (int i = 0; i < LOG_SIZE; i++) {           // перебираем импульсы
  Serial.println(durus[l][i]); 
  }
  Serial.print("duration "); 
  Serial.println( durus[l][LOG_SIZE]);
  
  Serial.print("count "); 
  Serial.println( durus[l][LOG_SIZE+1]);

  Serial.print("Канал "); 
  Serial.println( durus[l][LOG_SIZE+2]);

  Serial.print("Усиление "); 
  Serial.println( durus[l][LOG_SIZE+3]);

  Serial.print("Импульсов "); 
  Serial.println( durus[l][LOG_SIZE+4]);

  Serial.print("Позиция "); 
  Serial.println( durus[l][LOG_SIZE+5]);
  Serial.println();
  } 

  Serial.print("AccelX = "); 
  Serial.println(geo[0]);

  Serial.print("AccelY = "); 
  Serial.println(geo[1]);

  Serial.print("AccelZ = "); 
  Serial.println(geo[2]);

  Serial.print("MagX = "); 
  Serial.println(geo[3]);

  Serial.print("MagY = "); 
  Serial.println(geo[4]);

  Serial.print("MagZ = "); 
  Serial.println(geo[5]);

  Serial.print("Pressure = "); 
  Serial.println(geo[6]);

  Serial.print("Altitude = "); 
  Serial.println(geo[7]);
  Serial.println();

  Serial.println("Массив прочитан");
  Serial.println();
  } 


  void setup() {

  pinMode(STR_Pin, OUTPUT);             // назначение вывода STR_Pin выходом 
  pinMode(FO_Pin, OUTPUT);              // назначение вывода FO_Pin выходом 
  pinMode(DC_Pin, OUTPUT);              // назначение вывода DC_Pin выходом 
  pinMode(A0R_Pin, OUTPUT);             // назначение вывода A0R_Pin выходом 
  pinMode(A1R_Pin, OUTPUT);             // назначение вывода A1R_Pin выходом 

  pinMode(echoPinA, INPUT);             // назначение вывода echoA входом
  pinMode(echoPinB, INPUT);             // назначение вывода echoB входом
  pinMode(echoPinC, INPUT);             // назначение вывода echoC входом
  pinMode(echoPinD, INPUT);             // назначение вывода echoD входом 
  pinMode(analog1Pin, INPUT);           // назначение вывода analog1Pin входом 

  pinMode(START_Pin, INPUT);            // назначение вывода START_Pin входом 
  pinMode(PUSK_Pin, INPUT_PULLUP);      // назначение вывода PUSK_Pin входом
  pinMode(POZ_Pin, INPUT_PULLUP);       // назначение вывода POZ_Pin входом

                        // Конфигурация ШИМ
  ledc_timer_config_t ledc_timer;  
  ledc_timer.speed_mode   = LEDC_LOW_SPEED_MODE;
  ledc_timer.timer_num    = LEDC_TIMER_1;
  ledc_timer.duty_resolution = LEDC_TIMER_8_BIT;
  ledc_timer.freq_hz      = pwmfrequency;
  ledc_channel_config_t ledc_channel2;
  ledc_channel2.channel    = LEDC_CHANNEL_2;
  ledc_channel2.gpio_num   = STR_Pin;	// GPIO16 STR_Pin
  ledc_channel2.speed_mode = LEDC_LOW_SPEED_MODE;
  ledc_channel2.timer_sel  = LEDC_TIMER_1;
  ledc_channel2.duty       = dutyCycle;
  ledc_timer_config(&ledc_timer);
  ledc_channel_config(&ledc_channel2);
  ledc_fade_func_install(ESP_INTR_FLAG_SHARED);

       // конфигурируем функционирование LEDC PWM   
  ledcSetup(ledChannel0, freq0, resolution0);     // Задаём номер канала, частоту, скважность

       // attach the channel to the GPIO to be controlled
  ledcAttachPin(ledPin0, ledChannel0);            // Привязываем номер вывода к номеру канала

                        // Запускаем монитор порта
  Serial.begin(115200);
  // while(!Serial);
  // delay(5000);
  Serial.println();

  SPI.begin();
                        // Выставляем режим работы Wi-Fi
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  WiFi.setTxPower(WIFI_POWER_MINUS_1dBm);         // Установка минимального уровня мощности WiFi

            // Устанавливаем новый MAC Address этой платы 
  esp_wifi_set_mac(WIFI_IF_STA, &newMACAddress[0]);

                // Инициализируем протокол ESP-NOW
  if (esp_now_init() != ESP_OK) {
  //Serial.println("Error initializing ESP-NOW");
  return;
  }
         // Регистрируем callback-функцию для получения статуса отправки
  esp_now_register_send_cb(OnDataSent);
 
              // Регистрируем пиры
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;

                // Добавляем пиры
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
  //Serial.println("Failed to add peer");
  return;
  }
        // Регистрируем callback-функцию для получения статуса приема
  esp_now_register_recv_cb(OnDataRecv);

  EEPROM.begin(EEPROM_SIZE);
  
  /*
  sxb = EEPROM.readFloat(0);
  syb = EEPROM.readFloat(4);
  szb = EEPROM.readFloat(8);
  sxs = EEPROM.readFloat(12);
  sys = EEPROM.readFloat(16);
  szs = EEPROM.readFloat(20);

  // Display bias/scale factors
  Serial.println("\nЧТЕНИЕ EEPROM\n");
  Serial.println("---- MAG BIAS VALUES ----\n");
  Serial.print("X_Bias : "); Serial.print(sxb); Serial.print(',');
  Serial.print("Y_Bias : "); Serial.print(syb); Serial.print(',');
  Serial.print("Z_Bias : "); Serial.println(szb);

  Serial.println("\n---- MAG SCALE VALUES ----\n");
  Serial.print("X_Scale : "); Serial.print(sxs); Serial.print(',');
  Serial.print("Y_Scale : "); Serial.print(sys); Serial.print(',');
  Serial.print("Z_Scale : "); Serial.println(szs);
  */

  exb = EEPROM.readFloat(40);
  eyb = EEPROM.readFloat(44);
  ezb = EEPROM.readFloat(48);
  exs = EEPROM.readFloat(52);
  eys = EEPROM.readFloat(56);
  ezs = EEPROM.readFloat(60);

  // Display bias/scale factors
  Serial.println("\nЧТЕНИЕ EEPROM\n");
  Serial.println("---- MAG BIAS VALUES ----\n");
  Serial.print("X_Bias : "); Serial.print(exb); Serial.print(',');
  Serial.print("Y_Bias : "); Serial.print(eyb); Serial.print(',');
  Serial.print("Z_Bias : "); Serial.println(ezb);

  Serial.println("\n---- MAG SCALE VALUES ----\n");
  Serial.print("X_Scale : "); Serial.print(exs); Serial.print(',');
  Serial.print("Y_Scale : "); Serial.print(eys); Serial.print(',');
  Serial.print("Z_Scale : "); Serial.println(ezs);


  // start communication with IMU 
  IMU.begin();
  // setting the accelerometer full scale range to +/-2G 
  IMU.setAccelRange(MPU9250::ACCEL_RANGE_2G);
  // setting DLPF bandwidth to 20 Hz
  IMU.setDlpfBandwidth(MPU9250::DLPF_BANDWIDTH_20HZ);
  // setting SRD to 19 for a 50 Hz update rate
  IMU.setSrd(19);

  /*
  IMU.setAccelCalX(sxb, sxs);
  IMU.setAccelCalY(syb, sys);
  IMU.setAccelCalZ(szb, szs);
  */

  IMU.setAccelCalX(0.14, 1);
  IMU.setAccelCalY(-0.14, 1);
  IMU.setAccelCalZ(0.72, 1);

  IMU.setMagCalX(exb, exs);
  IMU.setMagCalY(eyb, eys);
  IMU.setMagCalZ(ezb, ezs);
 
  bmp.begin(BMP280_ADDRESS_ALT, BMP280_CHIPID);

  /* Default settings from datasheet. */
  bmp.setSampling(Adafruit_BMP280::MODE_NORMAL,     /* Operating Mode. */
                  Adafruit_BMP280::SAMPLING_X2,     /* Temp. oversampling */
                  Adafruit_BMP280::SAMPLING_X16,    /* Pressure oversampling */
                  Adafruit_BMP280::FILTER_X16,      /* Filtering. */
                  Adafruit_BMP280::STANDBY_MS_500); /* Standby time. */
  }

  void CalMag() {

  IMU.setMagCalX(0, 1);
  IMU.setMagCalY(0, 1);
  IMU.setMagCalZ(0, 1);

  Serial.println("\nPress [ ENTER ] to calibrate megnetometer...");

  // Countdown timer
  for (int i = 5; i > 0; i--)
  {
  Serial.print(i);
  Serial.println("...");
  delay(1000);
  }

  if (IMU.calibrateMag() == true)
  {
  // Gather magnetometer bias/scale factors
  hxb = IMU.getMagBiasX_uT();
  hyb = IMU.getMagBiasY_uT();
  hzb = IMU.getMagBiasZ_uT();
  hxs = IMU.getMagScaleFactorX();
  hys = IMU.getMagScaleFactorY();
  hzs = IMU.getMagScaleFactorZ();

  EEPROM.writeFloat(40, hxb);
  EEPROM.writeFloat(44, hyb);
  EEPROM.writeFloat(48, hzb);
  EEPROM.writeFloat(52, hxs);
  EEPROM.writeFloat(56, hys);
  EEPROM.writeFloat(60, hzs);
  EEPROM.commit();

  // Display bias/scale factors
  Serial.println("\nCALIBRATION COMPLETE\n");
  Serial.println("---- MAG BIAS VALUES ----\n");
  Serial.print("X_Bias : "); Serial.print(hxb); Serial.print(',');
  Serial.print("Y_Bias : "); Serial.print(hyb); Serial.print(',');
  Serial.print("Z_Bias : "); Serial.println(hzb);

  Serial.println("\n---- MAG SCALE VALUES ----\n");
  Serial.print("X_Scale : "); Serial.print(hxs); Serial.print(',');
  Serial.print("Y_Scale : "); Serial.print(hys); Serial.print(',');
  Serial.print("Z_Scale : "); Serial.println(hzs);

  // Set biased/scale factors
  IMU.setMagCalX(hxb, hxs);
  IMU.setMagCalY(hyb, hys);
  IMU.setMagCalZ(hzb, hzs);
  Serial.println("\n---- MAGNOMETER BIAS/SCALE FACTOR SET TO ----\n");

  Serial.print("Mag X Bias : "); Serial.print(hxb); Serial.print("\tMag X Scale Factor : "); Serial.println(hxs);
  Serial.print("Mag Y Bias : "); Serial.print(hyb); Serial.print("\tMag Y Scale Factor : "); Serial.println(hys);
  Serial.print("Mag Z Bias : "); Serial.print(hzb); Serial.print("\tMag Z Scale Factor : "); Serial.println(hzs);
  }
  }

  void CalAccel() {

  IMU.setAccelCalX(0, 1);
  IMU.setAccelCalY(0, 1);
  IMU.setAccelCalZ(0, 1);

  Serial.println("========================================================================================================================");
  Serial.println("\n-------------[ CALIBRATING ACCELEROMOTOR ]-------------\n");
  delay(1200);
  Serial.println("Orientate IMU to desiered INITAL position to calibrate all SIX axes\n");
  delay(1200);
  Serial.println("To initiate ACCELEROMOTOR calibration press [ ENTER ]...");

  // Calibration won't start until user inputs 'ENTER' in the serial monitor
  while (true) {
  if (digitalRead(START_Pin) == 0) {		          // Если есть нажатие кнопки Enter
  Serial.println("\nCALIBRATION STARTING IN...");
  break;
  }
  Serial.print(".");
  delay(1000);
  }
  delay(1200);

  // This is where the actual calibration takes place. Orientate the IMU sensor in SIX different directions AND MAKE SURE IT'S STABLE
  for (int n = 1; n <= 6; n++)
  {
  Serial.print("\nCurrent calibration iteration(s) : ");
  Serial.print(n); Serial.println(" \\ 6\n");

  // Countdown timer
  for (int i = 5; i > 0; i--)
  {
  Serial.print(i);
  Serial.println("...");
  delay(1000);
  }
  Serial.println("\n-------------[ CALIBRATION IN PROGRESS ]-------------");

  IMU.calibrateAccel();

  Serial.println("\n-------------[ Change IMU posistion ]-------------");
  }
  Serial.println("\n-------------[ Calibrating accelerometer COMPLETE ]-------------\n");

  // Get Scale factor values
  // Scale factors are the ratio between the measured output and the change in the 'sense' input
  // More info about scale factors can be found here : https://www.edn.com/evaluating-inertial-measurement-units/
  axs = IMU.getAccelScaleFactorX();
  ays = IMU.getAccelScaleFactorY();
  azs = IMU.getAccelScaleFactorZ();

  // Biased Factors
  axb = IMU.getAccelBiasX_mss();
  ayb = IMU.getAccelBiasY_mss();
  azb = IMU.getAccelBiasZ_mss();

  /*
  EEPROM.writeFloat(0, axb);
  EEPROM.writeFloat(4, ayb);
  EEPROM.writeFloat(8, azb);
  EEPROM.writeFloat(12, axs);
  EEPROM.writeFloat(16, ays);
  EEPROM.writeFloat(20, azs);
  EEPROM.commit();
  */

  // Display scale/bias values
  Serial.println("\n-------------[ SCALE FACTORS ]-------------\n");

  Serial.print("X_Scale Factor : "); Serial.print(axs);
  Serial.print(" Y_Scale Factor : "); Serial.print(ays);
  Serial.print(" Z_Scale Factor : "); Serial.println(azs);

  Serial.println("\n-------------[ BIAS VALUES]-------------\n");

  Serial.print("X_Biased : "); Serial.print(axb);
  Serial.print(" Y_Biased : "); Serial.print(ayb);
  Serial.print(" Z_Biased : "); Serial.println(azb);

  // Store scale/bias values into an array
  const float accelBias[3] = {axb, ayb, azb};
  const float accelFactor[3] = {axs, ays, azs};

  // Set the scale/bias values accordingly
  Serial.println("\n-------------[ IMU VALUES SET TO]-------------\n");
  IMU.setAccelCalX(accelBias[0], accelFactor[0]);
  IMU.setAccelCalY(accelBias[1], accelFactor[1]);
  IMU.setAccelCalZ(accelBias[2], accelFactor[2]);

  // Display
  Serial.print("Ax_Bias : "); Serial.print(accelBias[0]);  Serial.print(" | Ax_Accel : "); Serial.println(accelFactor[0]);
  Serial.print("Ay_Bias : "); Serial.print(accelBias[1]);  Serial.print(" | Ay_Accel : "); Serial.println(accelFactor[1]);
  Serial.print("Az_Bias : "); Serial.print(accelBias[2]);  Serial.print(" | Az_Accel : "); Serial.println(accelFactor[2]);
  delay(3000);

  Serial.println("\n========================================================================================================================");
  delay(2000);

  }

  void QryPin () {
               // проверка кнопоки POZ на нажатие - управление счётчиком позиций
  pozknt =  digitalRead(POZ_Pin);
  if (pozknt == 0 ) {                         // Если 0, смотрим, первое чтение, или нет
  if (pozknp == 1 ) {                         // Если 1, первое чтение, 
  pozknpush =  millis();                      // Читаем текущее машинное время нажатия кнопки POZ_Pin
  pozkn = 1;                                  // устанавливаем флаг нажатия кнопки POZ_Pin
  } else {                                    // Если не первое чтение,
  pozknnow =  millis();                       // Читаем текущее машинное время в мс 
  if (pozkn && pozknnow - pozknpush >= 700){  // Если есть флаг нажатия и время нажатия больше 0,7 сек
  poz = 0;                                    // Сброс счётчика позиции
  pozkn = 0; } }                              // сброс флага нажатия кнопки POZ_Pin
  } else                                      // если кнопку отжали
  if (pozknp == 0 ) {                         // Если 0, отжатие 
  if (pozkn == 1 ) {                           // Если 0, отжатие 
  poz = poz + 1;                              // Увеличение счётчика позиции
  pozkn = 0;                                  // сброс флага нажатия кнопки POZ_Pin
 } }  
  pozknp = pozknt;                            // записываем текущее состояние кнопки в предыдущее

               // проверка кнопоки PUSK на нажатие - запись на карту памяти
  flpuskt =  digitalRead(PUSK_Pin);
  if (flpuskt == 0 ) {                         // Если 0, смотрим, первое чтение, или нет
  if (flpuskp == 1 ) {                         // Если 1, первое чтение, 
  bitSet(regch, 6); }                          // запись единицы в 6 разряд 
  } 
  flpuskp = flpuskt;                           // записываем текущее состояние кнопки в предыдущее

               // проверка кнопоки START на нажатие - калибровка компаса
  flstart =  digitalRead(START_Pin);
  if (flstart == 0 ) {                         // Если 0, смотрим, первое чтение, или нет
  if (flstarp == 1 ) {                         // Если 1, первое чтение, 
  starknpush = millis();                       // Читаем текущее машинное время нажатия кнопки START_Pin
  flstar = 1;                                  // устанавливаем флаг нажатия кнопки START_Pin
  } else {                                     // Если не первое чтение,
  starknnow =  millis();                       // Читаем текущее машинное время в мс 
  if (flstar && starknnow - starknpush >= 700){   // Если есть флаг нажатия и время нажатия больше 0,7 сек
  flAccel = 1;                                 // устанавливаем флаг калибровки акселерометра
  flstar = 0; } }                              // сброс флага нажатия кнопки START_Pin
  } else {                                      // если кнопку отжали
  if (flstarp == 0 ) {                          // Если 0, отжатие 
  if (flstar == 1 ) {                          // Если было нажатие 
  flMag = 1;                                   // устанавливаем флаг калибровки компаса
  flstar = 0;                                  // сброс флага нажатия кнопки START_Pin
  } } } 
  flstarp = flstart;                           // записываем текущее состояние кнопки в предыдущее
  }
 
  void loop() {

  if (flagrec == 1) {                            // если пришла посылка от WeMosArd
  
                     // Чтение данных от приёмника
                     // читаем время и текущий пин 

  now = micros();                                // Читаем текущее машинное время

                  // Обрабатываем первый импульс канала D
  if (countD < LOG_SIZE) {                       // Если буфер не переполнен
  bool sD = digitalRead(echoPinD);               // Читаем текущее состояние пина
  if (prevSD != sD) {                            // Состояние пина изменилось?
  prevSD = sD;                                   // Если изменилось, запомнили текущее состояние
  if (!flagD) {                                  // Опрос флага (первое срабатывание?)
  flagD = 1;                                     // Если первое срабатывание, устанавливаем флаг в 1
  us[0][countD] = now - start;                   // время до первого импульса
  durus[0][countD] = us[0][countD];              // заносим время до первого импульса в durus[count]
  durationD = durus[0][countD];                  // duration = время до фронта первого импульса
  prevUsD = now;                                 // заносим текущее машинное время в prevUs
  countD++;                                      // + 1 в счётчик циклов
  } else {                                       // Если не первое срабатывание

                  // Обрабатываем следующие импульсы канала D
  us[0][countD] = now - prevUsD;                 // длительность импульса
  durus[0][countD] = (durus[0][countD - 1] + us[0][countD]);  // durus[0][count] = текущее время всех импульсов
  durationD = durus[0][countD - 1];              // duration = время до фронта последнего импульса
  durus[0][LOG_SIZE] = durationD;                // записываем duration
  durus[0][LOG_SIZE+1] = countD;                 // записываем count
  prevUsD = now;                                 // Запомнили машинное время
  countD++;                                      // + 1 в счётчик циклов 
  }}                                             // Если состояние пина не изменилось, идём дальше
  }                                              // Если буфер переполнен, ждём до окончания цикла 60 мс  
 
                  // Обрабатываем первый импульс канала C
  if (countC < LOG_SIZE) {                       // Если буфер не переполнен
  bool sC = digitalRead(echoPinC);               // Читаем текущее состояние пина
  if (prevSC != sC) {                            // Состояние пина изменилось?
  prevSC = sC;                                   // Если изменилось, запомнили текущее состояние
  if (!flagC) {                                  // Опрос флага (первое срабатывание?)
  flagC = 1;                                     // Если первое срабатывание, устанавливаем флаг в 1
  us[1][countC] = now - start;                   // время до первого импульса
  durus[1][countC] = us[1][countC];              // заносим время до первого импульса в durus[count]
  durationC = durus[1][countC];                  // duration = время до фронта первого импульса
  prevUsC = now;                                 // заносим текущее машинное время в prevUs
  countC++;                                      // + 1 в счётчик циклов
  } else {                                       // Если не первое срабатывание

                  // Обрабатываем следующие импульсы канала C
  us[1][countC] = now - prevUsC;                 // длительность импульса
  durus[1][countC] = (durus[1][countC - 1] + us[1][countC]);  // durus[1][count] = текущее время всех импульсов
  durationC = durus[1][countC - 1];              // duration = время до фронта последнего импульса
  durus[1][LOG_SIZE] = durationC;                // записываем duration
  durus[1][LOG_SIZE+1] = countC;                 // записываем count
  prevUsC = now;                                 // Запомнили машинное время
  countC++;                                      // + 1 в счётчик циклов 
  }}                                             // Если состояние пина не изменилось, идём дальше
  }                                              // Если буфер переполнен, ждём до окончания цикла 60 мс  

                  // Обрабатываем первый импульс канала B
  if (countB < LOG_SIZE) {                       // Если буфер не переполнен
  bool sB = digitalRead(echoPinB);               // Читаем текущее состояние пина
  if (prevSB != sB) {                            // Состояние пина изменилось?
  prevSB = sB;                                   // Если изменилось, запомнили текущее состояние
  if (!flagB) {                                  // Опрос флага (первое срабатывание?)
  flagB = 1;                                     // Если первое срабатывание, устанавливаем флаг в 1
  us[2][countB] = now - start;                   // время до первого импульса
  durus[2][countB] = us[2][countB];              // заносим время до первого импульса в durus[count]
  durationB = durus[2][countB];                  // duration = время до фронта первого импульса
  prevUsB = now;                                 // заносим текущее машинное время в prevUs
  countB++;                                      // + 1 в счётчик циклов
  } else {                                       // Если не первое срабатывание

                   // Обрабатываем следующие импульсы канала B
  us[2][countB] = now - prevUsB;                 // длительность импульса
  durus[2][countB] = (durus[2][countB - 1] + us[2][countB]);   // durus[2][count] = текущее время всех импульсов
  durationB = durus[2][countB - 1];              // duration = время до фронта последнего импульса
  durus[2][LOG_SIZE] = durationB;                // записываем duration
  durus[2][LOG_SIZE+1] = countB;                 // записываем count
  prevUsB = now;                                 // Запомнили машинное время
  countB++;                                      // + 1 в счётчик циклов 
  }}                                             // Если состояние пина не изменилось, идём дальше
  }                                              // Если буфер переполнен, ждём до окончания цикла 60 мс  

                  // Обрабатываем первый импульс канала A
  if (countA < LOG_SIZE) {                       // Если буфер не переполнен
  bool sA = digitalRead(echoPinA);               // Читаем текущее состояние пина
  if (prevSA != sA) {                            // Состояние пина изменилось?
  prevSA = sA;                                   // Если изменилось, запомнили текущее состояние
  if (!flagA) {                                  // Опрос флага (первое срабатывание?)
  flagA = 1;                                     // Если первое срабатывание, устанавливаем флаг в 1
  us[3][countA] = now - start;                   // время до первого импульса
  durus[3][countA] = us[3][countA];              // заносим время до первого импульса в durus[count]
  durationA = durus[3][countA];                  // duration = время до фронта первого импульса
  prevUsA = now;                                 // заносим текущее машинное время в prevUs
  countA++;                                      // + 1 в счётчик циклов
  } else {                                       // Если не первое срабатывание

                    // Обрабатываем следующие импульсы канала A
  us[3][countA] = now - prevUsA;                 // длительность импульса
  durus[3][countA] = (durus[3][countA - 1] + us[3][countA]);  // durus[3][count] = текущее время всех импульсов
  durationA = durus[3][countA - 1];              // duration = время до фронта последнего импульса
  durus[3][LOG_SIZE] = durationA;                // записываем duration
  durus[3][LOG_SIZE+1] = countA;                 // записываем count
  prevUsA = now;                                 // Запомнили машинное время
  countA++;                                      // + 1 в счётчик циклов 
  } }                                             // Если состояние пина не изменилось, идём дальше
  }                                              // Если буфер переполнен, ждём до окончания цикла 60 мс  
  if (now - start > 60000) {                     // Если время измерения истекло
  for (int l = 0; l < CHN_SIZE; l++) {           // перебираем каналы
  durus[l][LOG_SIZE+2] = l;                      // записываем канал D
  durus[l][LOG_SIZE+3] = u;                      // записываем канал C
  durus[l][LOG_SIZE+4] = p;                      // записываем канал B
  durus[l][LOG_SIZE+5] = poz;                    // записываем позицию
  }
  if (zapisy == 1) {                             // Если режим записи
  ReadGY();
  }
  QryPin ();
  if (flMag == 1 ) {                           // Если 1, калибровка
  CalMag();
  flMag = 0;
  }
  if (flAccel == 1 ) {                           // Если 1, калибровка
  CalAccel();
  flAccel = 0;
  }
  ledcWrite(ledChannel0, dutyCycle1);            // Останавливаем канал (0 на выходе)
  logOut();                                      // Переходим на вывод лога
  }                                              // Если время измерения не истекло
  }                                              // если не пришла посылка от WeMosArd
  }                                              // завершение void loop()
 
                          // Посылка строба ШИМ
  void setimp() {
  ledc_set_fade_with_step(LEDC_LOW_SPEED_MODE, LEDC_CHANNEL_2, target_duty, scale, cycle_num );
  ledc_set_duty(LEDC_LOW_SPEED_MODE, LEDC_CHANNEL_2, dutyCycle); 
  ledc_update_duty(LEDC_LOW_SPEED_MODE, LEDC_CHANNEL_2);
  ledc_fade_start(LEDC_LOW_SPEED_MODE, LEDC_CHANNEL_2, LEDC_FADE_WAIT_DONE);
  ledc_stop(LEDC_LOW_SPEED_MODE, LEDC_CHANNEL_2, idle_level);
  }

  void initial() {                               // Установка начальных значений 

  if (bitRead(regout,5) == 0) {                  // Если A0, то 
  p = 1;                                         // 8 импульсов
  } else                                         // Если 1, идём мимо
  if (bitRead(regout,3) == 0 ) {                 // Если 0, то 
  p = 2;                                         // 16 импульсов
  } else {                                       // Если 1, идём мимо
  p = 0;                                         // 4 импульса
  }
  if (bitRead(regout,2) == 0) {                  // Если A0, то 
  u = 0;                                         // усиление х1
  } else                                         // Если 1, идём мимо
  if (bitRead(regout,1) == 0 ) {                 // Если 0, то 
  u = 1;                                         // усиление х6
  } else {                                       // Если 1, идём мимо
  u = 2;                                         // усиление х36
  }
  if (bitRead(regout,6) == 1) {                  // Если процесс записи на карту памяти, то 
  zapisy = 1;                                    // устанавливаем флаг
  } else {                                       // Если 0, 
  zapisy = 0;                                    // сбрасываем флаг
  } 
  for (int l = 0; l < CHN_SIZE; l++) {           // перебираем каналы
  for (int i = 0; i < LOG_SIZE+5; i++) {         // перебираем массив
  durus[l][i] = 0; } }                           // обнуляем массив
  durationA = 0;                                 // duration = время до фронта последнего импульса = 0
  durationB = 0;                                 // duration = время до фронта последнего импульса = 0
  durationC = 0;                                 // duration = время до фронта последнего импульса = 0
  durationD = 0;                                 // duration = время до фронта последнего импульса = 0
  flagA = 0;                                     // обнуляем флаг канала A
  flagB = 0;                                     // обнуляем флаг канала B
  flagC = 0;                                     // обнуляем флаг канала C
  flagD = 0;                                     // обнуляем флаг канала D
  countA = 0;                                    // Счётчик циклов канала A = 0
  countB = 0;                                    // Счётчик циклов канала B = 0
  countC = 0;                                    // Счётчик циклов канала C = 0
  countD = 0;                                    // Счётчик циклов канала D = 0
  prevSA = digitalRead(echoPinA);                // Запомнили начальное состояние текущего пина канала A
  prevSB = digitalRead(echoPinB);                // Запомнили начальное состояние текущего пина канала B
  prevSC = digitalRead(echoPinC);                // Запомнили начальное состояние текущего пина канала C
  prevSD = digitalRead(echoPinD);                // Запомнили начальное состояние текущего пина канала D
  send_CKL();                                    // Выводим сигнал запуска излучателя
  }

  void logOut() {    // передача полученных результатов на WeMosArd

                           // Читаем напряжение
  analog1 = analogRead(analog1Pin);              // читаем напряжение источника питания
  voltage1 = (analog1*20.9) / 4095;              // пересчитываем число в вольты 

  printSendingData();      // печать отправляемых данных
                       // Записываем переменные для отправки
  ISPReadings.regin = regch;
  ISPReadings.voltbat = voltage1;                // напряжение батареи
  for (int l = 0; l < CHN_SIZE; l++) {           // перебираем каналы
  for (int i = 0; i < LOG_SIZE+6; i++) {         // перебираем импульсы
  ISPReadings.arrey1[l][i] = durus[l][i];        // массив измерений
  } }
  for (int k = 0; k < GEO_SIZE; k++) {           // перебираем каналы
  ISPReadings.arrey2[k] = geo[k];                // массив измерений
  }
                                // Отправляем сообщение
  esp_now_send(broadcastAddress,(uint8_t*)&ISPReadings, sizeof(ISPReadings));
  flagrec = 0;                                   // сброс флага посылки 
  regch = 0;
  }

  void send_CKL() {           // выдача сигналов управления на исполнительное устройство
  			// Установка коэффициента усиления и DC
  digitalWrite(A0R_Pin, bitRead(regout, 2));     // запись состояния A0 в A0R_Pin
  digitalWrite(A1R_Pin, bitRead(regout, 1));     // запись состояния A1 в A1R_Pin
  digitalWrite(DC_Pin, !bitRead(regout, 4));     // запись состояния DC в DC_Pin

  			// Выдача частоты и строба
  if (bitRead(regout,5) == 0) {                  // Если A0, то коэф. = 1
  freqCLK = 10000;                               // Задаём частоту для 8 импульсов
  } else                                         // Если 1, идём мимо
  if (bitRead(regout,3) == 0 ) {                 // Если 0, то 16 импульсов
  freqCLK = 5000;                                // Задаём частоту для 16 импульсов
  } else {                                       // Если 1, идём мимо
  freqCLK = 20000;                               // Задаём частоту для 4 импульсов
  }
  ledcWrite(ledChannel0, dutyCycle0);            // Запускаем частоту 80 кГц
  ledc_set_freq(LEDC_LOW_SPEED_MODE, LEDC_TIMER_1, freqCLK); // Устанавливаем частоту
  setimp();                                      // Запускаем строб
  start = micros();
  }

  void ReadGY() {

  // read the sensor
  IMU.readSensor();
  delay(40);
  IMU.readSensor();

  // display the data
    Serial.print(F("AccelX = "));
    Serial.print(IMU.getAccelX_mss());
    Serial.println(" m/ss");
    geo[0] = (IMU.getAccelX_mss());

    Serial.print(F("AccelY = "));
    Serial.print(IMU.getAccelY_mss());
    Serial.println(" m/ss");
    geo[1] = (IMU.getAccelY_mss());

    Serial.print(F("AccelZ = "));
    Serial.print(IMU.getAccelZ_mss());
    Serial.println(" m/ss");
    Serial.println();
    geo[2] = (IMU.getAccelZ_mss());

    Serial.print(F("MagX = "));
    Serial.print(IMU.getMagX_uT());
    Serial.println(" uT");
    geo[3] = (IMU.getMagX_uT());

    Serial.print(F("MagY = "));
    Serial.print(IMU.getMagY_uT());
    Serial.println(" uT");
    geo[4] = (IMU.getMagY_uT());

    Serial.print(F("MagZ = "));
    Serial.print(IMU.getMagZ_uT());
    Serial.println(" uT");
    geo[5] = (IMU.getMagZ_uT());

    // Direction = atan2(IMU.getMagX_uT(),IMU.getMagY_uT())*180/Pi;

    // Serial.print(F("Direction = "));
    // Serial.print(Direction);
    // Serial.println(" grad");
    Serial.println();

    // Serial.print(F("Temperature = "));
    // Serial.print(bmp.readTemperature());
    // Serial.println(" *C");

    Serial.print(F("Pressure = "));
    Serial.print(bmp.readPressure());
    Serial.println(" Pa");
    geo[6] = (bmp.readPressure());

    Serial.print(F("Approx altitude = "));
    Serial.print(bmp.readAltitude(1013.25)); 
    Serial.println(" m");
    Serial.println();
    geo[7] = (bmp.readAltitude(1013.25));

    Serial.println();
    // delay(50);
  }


  /*            7       6       5       4       3       2       1       0        

     regm1     X36      X6      X1   12/24V     -    16 имп   8 имп   4 имп                          
    
     reg_IN     -       -       -       D       C       B       A       -      

     regkn2   ЭКРАН     -      АВТО     D       C       B       A     СРЕДА 

       -        -       -       -      L1       L2      L3     L4       -        

       -        -       -       -     КАН 4   КАН 3    КАН 2   КАН 1    -          

     regout     -     zapisy   A0T     DC      A1T     A0R     A1R      -                           
        0x3A                    1       1       1       0       1       0 


      regch     -     pusk      -       -       -       -       -       -


                               A0T   DC_Pin    A1T   A0R_Pin  A1R_Pin   0 
    активный уровень            0       1       0       0       0       0
  */                                        

 
